package top.daxian1218.dust.service.repository;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.mongodb.client.FindIterable;
import com.mongodb.client.ListIndexesIterable;
import com.mongodb.client.MongoCollection;
import com.mongodb.client.MongoCursor;
import com.mongodb.client.model.FindOneAndUpdateOptions;
import com.mongodb.client.model.IndexOptions;
import com.mongodb.client.model.ReturnDocument;
import com.mongodb.client.model.UpdateOptions;
import com.mongodb.client.result.UpdateResult;
import org.apache.commons.lang3.StringUtils;
import org.bson.Document;
import org.bson.types.ObjectId;
import org.springframework.data.mongodb.core.MongoTemplate;
import org.springframework.data.mongodb.core.query.Criteria;
import org.springframework.data.mongodb.core.query.Query;
import top.daxian1218.dust.commons.util.TypeUtil;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

public class MongoRepository {
    MongoTemplate mongoTemplate;

    private final String DUST_LOCKS = "dust_locks";

    public MongoRepository(MongoTemplate template) {
        this.mongoTemplate = template;
    }

    private Document toDBObject(Object data) {
        if (data == null) {
            return new Document();
        }

        if (data instanceof  Document){
            return  (Document) data;
        }

        if (data instanceof JSONObject) {
            return  Document.parse(((JSONObject) data).toJSONString());
        }

        if (data instanceof String) {
            return  Document.parse((String) data);
        }

        return Document.parse(JSON.toJSONString(data));
    }

    public String[] insert(String collectionName, Object data) {
        List<Document> docs = new ArrayList();
        if (data instanceof List) {
            ((List) data).forEach(o -> {
                Document doc = toDBObject(o);
                unifyInsert(doc);
                docs.add(doc);
            });

            if (docs.size() == 0) {
                return new String[]{};
            }

        } else {
            Document doc = toDBObject(data);
            unifyInsert(doc);
            docs.add(doc);
        }

        MongoCollection<Document> dbCollection = mongoTemplate.getCollection(collectionName);
        dbCollection.insertMany(docs);
        return docs.stream().map((doc)-> ((ObjectId)doc.get("_id")).toHexString()).toArray(String[]::new);
    }

    public int update(String collectionName, String id, Object data) {
        Document doc = data == null ? new Document() : toDBObject(data);
        unifyUpdate(doc);
        Document update = new Document("$set", doc);

        Document query = new Document("_id", new ObjectId(id));
        MongoCollection<Document> dbCollection = mongoTemplate.getCollection(collectionName);
        UpdateResult result = dbCollection.updateOne(query, update);
        return (int)result.getModifiedCount();
    }

    public int updateOrInsert(String collectionName, Object query, Object update) {
        Document doc = update == null ? new Document() : toDBObject(update);
        unifyUpdate(doc);

        Document updateDoc = new Document("$set", doc);

        MongoCollection<Document> dbCollection = mongoTemplate.getCollection(collectionName);
        UpdateOptions opts = new UpdateOptions();
        opts.upsert(true);
        UpdateResult result = dbCollection.updateMany(toDBObject(query), updateDoc, opts);
        return (int)result.getModifiedCount();
    }

    public int update(String collectionName, Object query, Object update) {
        Document doc = update == null ? new Document() : toDBObject(update);
        unifyUpdate(doc);

        Document updateDoc = new Document("$set", doc);

        MongoCollection<Document> dbCollection = mongoTemplate.getCollection(collectionName);
        UpdateResult result = dbCollection.updateMany(toDBObject(query), updateDoc);
        return (int)result.getModifiedCount();
    }

    public boolean updateStrict(String collectionName, Object query, Object update) {
        Document doc = update == null ? new Document() : toDBObject(update);
        unifyUpdate(doc);
        Document updateDoc = new Document("$set", doc);

        Document queryDoc = toDBObject(query);
        if (queryDoc.containsKey("id")) {
            queryDoc.put("_id", queryDoc.get("id"));
            queryDoc.remove("id");
        }

        MongoCollection<Document> dbCollection = mongoTemplate.getCollection(collectionName);
        Document old = dbCollection.findOneAndUpdate(queryDoc, updateDoc);
        return old != null;
    }

    public JSONArray find(String collectionName, Object query) {
        Document queryDoc = toDBObject(query);
        JSONArray data = new JSONArray();
        MongoCollection<Document> dbCollection = mongoTemplate.getCollection(collectionName);
        MongoCursor<Document> cursor = dbCollection.find(queryDoc).iterator();
        while (cursor.hasNext()) {
            Document doc =  cursor.next();
            JSONObject row = doc2Json(doc);
            row.put("id", row.getString("_id"));
            row.remove("_id");
            data.add(row);
        }
        cursor.close();
        return data;
    }

    public JSONObject findAndModify(String collectionName, Object query, Object inc, Object push) {
        Document queryDoc = toDBObject(query);
        Document update = new Document();
        if (inc != null) {
            update.put("$inc", toDBObject(inc));
        }

        if (push != null) {
            update.put("$push", toDBObject(push));
        }
        MongoCollection<Document> dbCollection = mongoTemplate.getCollection(collectionName);
        FindOneAndUpdateOptions options = new FindOneAndUpdateOptions();
        options.upsert(true);
        options.returnDocument(ReturnDocument.AFTER);
        Document doc = dbCollection.findOneAndUpdate(queryDoc, update, options);
        if (doc == null) {
            return null;
        }
        return doc2Json(doc);
    }

    public JSONObject findOne(String collectionName, String id) {
        MongoCollection<Document> dbCollection = mongoTemplate.getCollection(collectionName);
        Document doc = dbCollection.find(new Document("_id", new ObjectId(id))).first();
        if (doc == null) {
            return null;
        }
        return doc2Json(doc);
    }

    public void drop(String collectionName) {
        MongoCollection<Document> dbCollection = mongoTemplate.getCollection(collectionName);
        dbCollection.drop();
    }


    public boolean hasLock(String lockName, Object lockValue) {
        MongoCollection<Document> dbCollection = mongoTemplate.getCollection(DUST_LOCKS);
        Document doc = dbCollection.find(new Document("lockName", lockName).append("lockValue", lockValue)).first();
        if (doc == null) {
            return false;
        }
        return true;
    }


    public boolean lock(String lockName, Object lockValue, Object newLockValue) {
        MongoCollection<Document> lockCollection = mongoTemplate.getCollection(DUST_LOCKS);
        FindIterable<Document> lock = lockCollection.find(new Document("lockName", lockName));
        if (lock == null) {
            try {
                lockCollection.insertOne(new Document("lockName", lockName).append("lockValue", newLockValue));
                return true;
            } catch (Exception ex) {
                //插入失败，继续执行更新操作
            }
        }

        Document query = new Document("lockName", lockName).append("lockValue", lockValue);
        Document update = new Document("lockValue", newLockValue);
        return updateStrict(DUST_LOCKS, query, update);
    }

    public boolean lock(String lockName, Object lockValue, boolean increase) {
        MongoCollection<Document> dbCollection = mongoTemplate.getCollection(DUST_LOCKS);
        Document update = new Document("$inc", new Document("lockValue", 1));
        update.put("$set", new Document("gmtModified", new Date().getTime()));

        Document oldDoc = dbCollection.findOneAndUpdate(new Document("lockName", lockName).append("lockValue", lockValue),
                update);
        if (oldDoc == null) {
            return false;
        }
        return true;
    }

    public void createIndex(String collectionName, String key, boolean unique) {
        MongoCollection<Document> dbCollection = mongoTemplate.getCollection(collectionName);
        ListIndexesIterable<Document> indexInfoList = dbCollection.listIndexes();
        for (Document doc : indexInfoList) {
            if (((Document)doc.get("key")).containsKey(key)) {
                return;
            }
        }

        Document keys = new Document(key, 1);
        dbCollection.createIndex(keys, new IndexOptions().unique(true));
    }

    public void delete(String collectionName, Object query) {
        Document queryDoc = toDBObject(query);
        MongoCollection<Document> dbCollection = mongoTemplate.getCollection(collectionName);
        dbCollection.deleteMany(queryDoc);
    }

    private Document unifyInsert(Document object) {
        object.put("gmtCreate", new Date().getTime());
        object.put("gmtModified", new Date().getTime());
        if (object.containsKey("id")) {
            object.remove("id");
        }
        return object;
    }

    private Document unifyUpdate(Document object) {
        object.put("gmtModified", new Date().getTime());
        if (object.containsKey("id")) {
            object.remove("id");
        }

        return object;
    }

    private JSONObject doc2Json(Document doc) {
        JSONObject json = new JSONObject();
        doc.forEach((k, v) -> {
            if (StringUtils.equals("_id", k)) {
                json.put(k, v.toString());
            } else {
                json.put(k, v);
            }
        });
        return json;
    }

    private JSONArray cursor2Array(MongoCursor<Document> cursor) {
        JSONArray arr = new JSONArray();
        cursor.forEachRemaining(d -> {
            arr.add(doc2Json(d));
        });
        return arr;
    }

    private JSONArray cursor2ArraySafe(MongoCursor<Document> cursor) {
        try {
            JSONArray arr = new JSONArray();
            cursor.forEachRemaining(d -> {
                arr.add(doc2Json(d));
            });
            return arr;
        } finally {
            cursor.close();
        }
    }

    private Document json2Doc(JSONObject json) {
        Document doc = new Document();
        json.forEach((key, val) -> {
            if (StringUtils.equals("_id", key) ||
                    StringUtils.equals("id", key)) {
                if (val == null) {
                    return;
                }

                doc.put("_id", new ObjectId(TypeUtil.toString(val)));
            } else {
                doc.put(key, val);
            }
        });


        return doc;
    }

    private List<Document> json2DocList(JSONArray arr) {
        List<Document> docs = new ArrayList();

        arr.forEach((o) -> {
            JSONObject jo = (JSONObject) o;
            docs.add(json2Doc(jo));
        });
        return docs;
    }
}
