package top.daxian1218.dust.service.security;

import com.alibaba.fastjson.JSONObject;
import org.apache.commons.lang3.StringUtils;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.GrantedAuthority;
import org.springframework.security.core.authority.SimpleGrantedAuthority;
import org.springframework.security.core.context.SecurityContext;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.security.core.userdetails.UserDetails;

import javax.swing.text.html.Option;
import java.security.Principal;
import java.util.Collection;
import java.util.Collections;
import java.util.Optional;

/**
 * 安全信息公共类，用于业务获取特定的信息
 * @author hst
 */
public final class SecurityUtils {

    private SecurityUtils() {
    }

    /**
     * 获取当前请求的用户
     *
     * @return 默认是userId
     */
    public static String getCurrentUser() {
        SecurityContext securityContext = SecurityContextHolder.getContext();
        Authentication authentication = securityContext.getAuthentication();
        if (authentication != null) {
            return authentication.getName();
        }
        return null;
    }

    public static String getUserId() {
         DustUser user = getUserInfo();
         if (user != null) {
             return user.getId();
         }

         return StringUtils.EMPTY;
    }

    /**
     * 是否匿名用户
     *
     * @return
     */
    public static boolean isAnonymous() {
        SecurityContext securityContext = SecurityContextHolder.getContext();
        Authentication authentication = securityContext.getAuthentication();
        if (authentication == null) {
            return true;
        }

        Collection<? extends GrantedAuthority> authorities = authentication.getAuthorities();
        if (authorities == null || authorities.size() == 0) {
            return true;
        }

        for (GrantedAuthority authority : authorities) {
            if (authority.getAuthority().equals(AuthoritiesConstants.ANONYMOUS)) {
                return true;
            }
        }

        return false;
    }

    /**
     * 是否是超级管理员
     * @return
     */
    public static boolean isSuperAdmin() {
        SecurityContext securityContext = SecurityContextHolder.getContext();
        Authentication authentication = securityContext.getAuthentication();
        if (authentication != null && authentication.getAuthorities() != null) {
            Collection<? extends GrantedAuthority> authorities = authentication.getAuthorities();
            for (GrantedAuthority authority : authorities) {
                if (authority.getAuthority().equals(AuthoritiesConstants.ADMIN)) {
                    return true;
                }
            }
       }
        return false;
    }

    public static boolean isDevelop() {
        return hasRole(AuthoritiesConstants.DEVELOP);
    }

    /**
     * 当前用户是否具有authority角色
     *
     *
     * @param authority the authority to check
     * @return true 具有相应的角色，否则 false
     */
    public static boolean hasRole(String authority) {
        SecurityContext securityContext = SecurityContextHolder.getContext();
        Authentication authentication = securityContext.getAuthentication();
        if (authentication != null) {
            return authentication.getAuthorities().stream()
                    .anyMatch(ga -> StringUtils.equals(ga.getAuthority(), authority));
        }
        return false;
    }

    /**
     * JWT以及业务接口组成的登录用户信息
     * NOTE： 只有使用{@link DustAuthentication}时，才会返回信息
     * @return json格式的内容
     */
    public static DustUser getUserInfo() {
        SecurityContext securityContext = SecurityContextHolder.getContext();
        Authentication authentication = securityContext.getAuthentication();
        if (authentication != null && authentication instanceof DustAuthentication) {
            return ((DustAuthentication) authentication).getUser();
        }
        return null;
    }

    public static void setUser(DustUser user) {
        SecurityContextHolder.getContext().setAuthentication(new DustAuthentication(user));
    }
}
