package top.daxian1218.dust.service.security.jwt;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONException;
import io.jsonwebtoken.Claims;
import io.jsonwebtoken.Jwts;
import io.jsonwebtoken.SignatureAlgorithm;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.context.SecurityContextHolder;
import top.daxian1218.dust.commons.thread.LocalHolder;
import top.daxian1218.dust.commons.util.ClassUtil;
import top.daxian1218.dust.commons.util.TypeUtil;
import top.daxian1218.dust.service.config.DustMsProperties;
import top.daxian1218.dust.service.security.DustAuthentication;
import top.daxian1218.dust.service.security.DustUser;

import javax.servlet.http.HttpServletRequest;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

/**
 * @author hst
 */
public class JwtAuthenticationImpl implements IAuthentication {
    private final Logger logger = LoggerFactory.getLogger(JwtAuthenticationImpl.class);

    private static final String TIMESTAMP = "timestamp";
    private static final String APP_ID = "appId";
    private static final String TENANT_ID = "tenantId";

    protected DustMsProperties dustMsProperties;

    public JwtAuthenticationImpl(DustMsProperties dustMsProperties) {
        this.dustMsProperties = dustMsProperties;
    }

    @Override
    public boolean validateRequest(HttpServletRequest request) {
        if (!isEnable()) {
            return true;
        }

        return this.validateRequestByDust(request);
    }

    public boolean validateRequestByDust(HttpServletRequest request) {
        DustAuthentication auth = null;
        String token = resolveToken(request);
        if (StringUtils.isEmpty(token)) {
            return false;
        }

        //JSON明文传输
        if (isJsonToken() && token.startsWith("{") && token.endsWith("}")) {
            auth = json2Authentication(token);
        } else {
            auth = bearer2Authentication(token);
        }

        if (auth == null) {
            return false;
        }

        SecurityContextHolder.getContext().setAuthentication(auth);
        this.resolveSysParam(request, auth.getUser());
        return true;
    }


    @Override
    public String createToken(Authentication authentication, Boolean rememberMe) {
        long now = (new Date()).getTime();
        Date validity;
        if (rememberMe) {
            validity = new Date(now + this.getTokenValidityInSecondsForRememberMe());
        } else {
            validity = new Date(now + this.getTokenValidityInSeconds());
        }
        Map<String, Object> claims = null;
        if (authentication instanceof DustAuthentication) {
            DustAuthentication dustAuthentication = (DustAuthentication) authentication;
            claims = dustAuthentication.getUser().toClaims();
            claims.put("sub", dustAuthentication.getUser().getId());
        } else {
            claims = new HashMap<>();
            claims.put("sub", authentication.getName());
            claims.put("roles", authentication.getAuthorities().stream()
                    .map(ga -> ga.getAuthority())
                    .reduce(StringUtils.EMPTY, (a, b) -> a + b));

            if (authentication.getDetails() != null) {
                claims.put("details", authentication.getDetails().toString());
            }
        }

        claims.put("issuer", "DUST");
        return Jwts.builder()
                .setSubject(authentication.getName())
                .setClaims(claims)
                .signWith(SignatureAlgorithm.HS512, getSecretKey())
                .setExpiration(validity)
                .compact();
    }

    @Override
    public String createRefreshToken(Authentication authentication) {
        long now = (new Date()).getTime();
        Date validity = new Date(now + this.getTokenValidityInSeconds() * 2);
        Map<String, Object> claims = null;
        if (authentication instanceof DustAuthentication) {
            DustAuthentication dustAuthentication = (DustAuthentication) authentication;
            claims = new HashMap<>();;
            claims.put("sid", dustAuthentication.getUser().getSid());
            claims.put("sub", dustAuthentication.getUser().getId());
            claims.put("tenantId", dustAuthentication.getUser().getTenantId());
            claims.put("appId", dustAuthentication.getUser().getAppId());
        } else {
            claims = new HashMap<>();
            claims.put("sub", authentication.getName());
        }

        claims.put("issuer", "DUST");
        return Jwts.builder()
                .setSubject(authentication.getName())
                .setClaims(claims)
                .signWith(SignatureAlgorithm.HS512, getSecretKey())
                .setExpiration(validity)
                .compact();
    }


    /**
     * JSON转化为DustAuthentication
     *
     * @param token
     * @return
     */
    protected DustAuthentication json2Authentication(String token) {
        try {
            DustUser user = JSON.parseObject(token, DustUser.class);
            user.setToken(token);
            return new DustAuthentication(user);
        } catch (JSONException ex) {
            debuggerLog(ex.getMessage());
            LocalHolder.set(AUTHORIZE_ERROR, "token不满足JSON格式");
            return null;
        }
    }


    /**
     * token转化为Authorization对象
     *
     * @param token
     */
    protected DustAuthentication bearer2Authentication(String token) {
        try {
            Claims claims = Jwts.parser()
                    .setSigningKey(getSecretKey())
                    .parseClaimsJws(token)
                    .getBody();

            DustUser user = ClassUtil.mapToObject(claims, DustUser.class);
            user.setToken(token);
            return new DustAuthentication(user);
        } catch (Exception ex) {
            debuggerLog(ex.getMessage());
            LocalHolder.set(AUTHORIZE_ERROR, ex.getMessage());
            return null;
        }
    }

    protected String resolveToken(HttpServletRequest request) {
        String token = request.getHeader(AUTHORIZATION_HEADER);
        if (StringUtils.isEmpty(token)) {
            token = request.getParameter(AUTHORIZATION_PARAMETER);
        }

        if (!StringUtils.isEmpty(token) && token.startsWith(TOKEN_BEARER_PREFIX)) {
            token = token.substring(7);
        }

        return token;
    }

    protected void resolveSysParam(HttpServletRequest req, DustUser dustUser) {
        Map<String, String[]> maps = req.getParameterMap();
        maps.forEach((s, strings) -> {
            if (StringUtils.equalsIgnoreCase(s, APP_ID)) {
                dustUser.setAppId(strings != null && strings.length > 0 ? strings[0] : "");
            }

            if (StringUtils.equalsIgnoreCase(s, TENANT_ID)) {
                dustUser.setTenantId(strings != null && strings.length > 0 ? strings[0] : "");
            }

            if (StringUtils.equalsIgnoreCase(s, TIMESTAMP)) {
                dustUser.setTimestamp(TypeUtil.toLong(strings != null && strings.length > 0 ? strings[0] : ""));
            }
        });
    }

    public boolean isEnable() {
        return dustMsProperties.getSecurity().getAuthentication().getJwt().isEnable();
    }

    public String getSecretKey() {
        return dustMsProperties.getSecurity().getAuthentication().getJwt().getSecret();
    }

    public long getTokenValidityInSeconds() {
        return 1000 * dustMsProperties.getSecurity().getAuthentication().getJwt().getTokenValidityInSeconds();
    }

    public long getTokenValidityInSecondsForRememberMe() {
        return 1000 * dustMsProperties.getSecurity().getAuthentication().getJwt().getTokenValidityInSecondsForRememberMe();
    }

    public boolean isJsonToken() {
        return dustMsProperties.getSecurity().getAuthentication().getJwt().isJson();
    }

    private void debuggerLog(String msg) {
        if (logger.isDebugEnabled()) {
            logger.debug(msg);
        }
    }
}
