package top.daxian1218.dust.service.security.jwt;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONException;
import io.jsonwebtoken.Claims;
import io.jsonwebtoken.Jwts;
import io.jsonwebtoken.SignatureAlgorithm;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.security.core.context.SecurityContextHolder;
import top.daxian1218.dust.commons.thread.LocalHolder;
import top.daxian1218.dust.commons.util.Base64Util;
import top.daxian1218.dust.commons.util.ClassUtil;
import top.daxian1218.dust.commons.util.TypeUtil;
import top.daxian1218.dust.service.config.DustMsProperties;
import top.daxian1218.dust.service.security.DustAuthentication;
import top.daxian1218.dust.service.security.DustUser;

import javax.servlet.http.HttpServletRequest;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

import static top.daxian1218.dust.service.security.jwt.IAuthentication.*;

/**
 * Token处理类
 * 使用jjwt包来进行token相关的转码操作
 *
 * @author hst
 */
public class TokenProvider {

    private final Logger logger = LoggerFactory.getLogger(TokenProvider.class);

    private static final String TIMESTAMP = "timestamp";
    private static final String APP_ID = "appId";
    private static final String TENANT_ID = "tenantId";

    private static final String ISSUER = "DUST";

    private DustMsProperties dustMsProperties;


    public  TokenProvider(DustMsProperties dustMsProperties) {
        this.dustMsProperties = dustMsProperties;
    }


    public String createJwtToken(DustUser user, Boolean rememberMe) {
        return createToken(user, rememberMe);
    }

    public boolean validateRequest(HttpServletRequest request) {
        if (!isEnable()) {
            return true;
        }

        return this.validateRequestByDust(request);
    }

    public boolean validateRequestByDust(HttpServletRequest request) {
        DustAuthentication auth = null;
        String token = resolveToken(request);
        if (StringUtils.isEmpty(token)) {
            return false;
        }

        //JSON明文传输
        if (isJsonToken() && token.startsWith("{") && token.endsWith("}")) {
            auth = json2Authentication(token);
        } else {
            auth = bearer2Authentication(token);
        }

        if (auth == null) {
            return false;
        }

        SecurityContextHolder.getContext().setAuthentication(auth);
        this.resolveSysParam(request, auth.getUser());
        return true;
    }

    public String createToken(DustUser user, Boolean rememberMe) {
        long now = (new Date()).getTime();
        Date validity;
        if (rememberMe) {
            validity = new Date(now + this.getTokenValidityInSecondsForRememberMe());
        } else {
            validity = new Date(now + this.getTokenValidityInSeconds());
        }
        Map<String, Object> claims = user.toClaims();
        claims.put("sub", user.getId());

        claims.put("issuer", ISSUER);
        return Jwts.builder()
                .setSubject(user.getId())
                .setClaims(claims)
                .signWith(SignatureAlgorithm.HS512, getSecretKey())
                .setExpiration(validity)
                .compact();
    }

    public String createRefreshToken(DustUser user) {
        long now = (new Date()).getTime();
        Date validity = new Date(now + this.getTokenValidityInSeconds() * 2);
        Map<String, Object> claims = new HashMap<>();;
        claims.put("sid", user.getSid());
        claims.put("sub", user.getId());
        claims.put("tenantId", user.getTenantId());
        claims.put("appId", user.getAppId());

        claims.put("issuer", ISSUER);
        return Jwts.builder()
                .setSubject(user.getId())
                .setClaims(claims)
                .signWith(SignatureAlgorithm.HS512, getSecretKey())
                .setExpiration(validity)
                .compact();
    }

    public DustUser convertToUser(String token)
    {
        DustAuthentication authentication = bearer2Authentication(token);
        return authentication.getUser();
    }


    /**
     * JSON转化为DustAuthentication
     *
     * @param token
     * @return
     */
    protected DustAuthentication json2Authentication(String token) {
        try {
            DustUser user = JSON.parseObject(token, DustUser.class);
            user.setToken(token);
            return new DustAuthentication(user);
        } catch (JSONException ex) {
            debuggerLog(ex.getMessage());
            LocalHolder.set(AUTHORIZE_ERROR, "token不满足JSON格式");
            return null;
        }
    }


    /**
     * token转化为Authorization对象
     *
     * @param token
     */
    protected DustAuthentication bearer2Authentication(String token) {
        try {
            Claims claims = Jwts.parser()
                    .setSigningKey(getSecretKey())
                    .parseClaimsJws(token)
                    .getBody();

            DustUser user = ClassUtil.mapToObject(claims, DustUser.class);
            user.setToken(token);
            return new DustAuthentication(user);
        } catch (Exception ex) {
            debuggerLog(ex.getMessage());
            LocalHolder.set(AUTHORIZE_ERROR, ex.getMessage());
            return null;
        }
    }

    protected String resolveToken(HttpServletRequest request) {
        String token = request.getHeader(AUTHORIZATION_HEADER);
        if (StringUtils.isEmpty(token)) {
            token = request.getParameter(AUTHORIZATION_PARAMETER);
        }

        if (!StringUtils.isEmpty(token) && token.startsWith(TOKEN_BEARER_PREFIX)) {
            token = token.substring(7);
        }

        return token;
    }

    protected void resolveSysParam(HttpServletRequest req, DustUser dustUser) {
        Map<String, String[]> maps = req.getParameterMap();
        maps.forEach((s, strings) -> {
            if (StringUtils.equalsIgnoreCase(s, APP_ID)) {
                dustUser.setAppId(strings != null && strings.length > 0 ? strings[0] : "");
            }

            if (StringUtils.equalsIgnoreCase(s, TENANT_ID)) {
                dustUser.setTenantId(strings != null && strings.length > 0 ? strings[0] : "");
            }

            if (StringUtils.equalsIgnoreCase(s, TIMESTAMP)) {
                dustUser.setTimestamp(TypeUtil.toLong(strings != null && strings.length > 0 ? strings[0] : ""));
            }
        });
    }

    public boolean isEnable() {
        return dustMsProperties.getSecurity().getAuthentication().getJwt().isEnable();
    }

    public String getSecretKey() {
        return dustMsProperties.getSecurity().getAuthentication().getJwt().getSecret();
    }

    public long getTokenValidityInSeconds() {
        return 1000 * dustMsProperties.getSecurity().getAuthentication().getJwt().getTokenValidityInSeconds();
    }

    public long getTokenValidityInSecondsForRememberMe() {
        return 1000 * dustMsProperties.getSecurity().getAuthentication().getJwt().getTokenValidityInSecondsForRememberMe();
    }

    public boolean isJsonToken() {
        return dustMsProperties.getSecurity().getAuthentication().getJwt().isJson();
    }

    private void debuggerLog(String msg) {
        if (logger.isDebugEnabled()) {
            logger.debug(msg);
        }
    }

}
